declare("POI_tab", {
	version = "1.0.1",
})
console_print("Point-of-Interest Tab for MDI-TargetList loaded (v"..POI_tab.version..").")

local T = {
	Tab = {
		title = "POI",
		columns = {
			{
				title = "SectorID",
				width = 0,
			},
			{
				title = "Description",
				width = 0.8,
			},
			{
				title = "Location",
				width = 0.2,
			},
		},
		filters = {
			{
				title = "All",
			},
			{
				title = "Wormholes",
			},
			{
				title = "Stations",
			},
			{
				title = "Other",
			},
		},
		--------------
		other_list = dofile("other_locations.lua"),
		poi_list = {},
		filter_index = -1,
	}
}
POI_tab.T = T

function T.Tab:ChangeTarget(oldID, newID)
	if (T.Tab.poi_list[newID]) then
		NavRoute.SetFinalDestination(T.Tab.poi_list[newID][1])
	end
end

function T.Tab:ChangeFilter(index, page)
	T.Tab.filter_index = index
	T.Tab:SECTOR_CHANGED(nil, GetCurrentSectorid(), true)
end

function T.Tab:TARGETLIST_TAB_CHANGED()
	self.is_active = targetlist.API.IsActiveTab(T.Tab)
	if (self.is_active) then
		--self:SECTOR_CHANGED(nil, GetCurrentSectorid(), true)
	end
end


function T.Tab:NAVROUTE_ADD(_, data)
	T.Tab:NAVROUTE_CHANGED(nil, data)
end

function T.Tab:NAVROUTE_CHANGED(_, data)
	if (T.Tab.is_active) then
		local sectorID = NavRoute.GetFinalDestination()
		-- can cause two list items to be highlighted if a station shares a sector with a wormhole
		for i, t in ipairs(T.Tab.poi_list) do
			t.highlight = t[1] == sectorID
		end
		targetlist.API.UpdateDisplay(self, self.poi_list)
	end
end

function T.Tab:NAVROUTE_UNDOLAST(_, data)
	T.Tab:NAVROUTE_CHANGED(nil, data)
end

function T.Tab:SECTOR_CHANGED(event, sectorID, filter_changed)
	if (self.is_active) then
		local systemID = GetSystemID(sectorID)
		if (systemID == self.current_systemID and not filter_changed) then
			return
		else
			self.current_systemID = systemID
		end
		-- Does this system have a training sector?
		local trainingID
		for i = 1, 3 do
			for _, startingID in ipairs(GetStartingSectors(i)) do
				if (GetSystemID(startingID) == GetCurrentSystemid()) then
					trainingID = startingID
					break
				end
			end
		end

		self.poi_list = {}
		for poi_type, pois in pairs(GetPOI()) do
			for i, poi_data in ipairs(pois) do
				local sectorID = poi_data.sectorid

				local insert_poi = false
				local desc
				if     (poi_type == "wormholes") then
					-- "Sol II System, Sector O-14 (Serco Monitored)|Wormhole: to Dantia|"
					desc = "Wormhole: to "..SystemNames[poi_data.destination] --sector_data.desc:match("(Wormhole:.*)|")
					insert_poi = self.filter_index == 1 or self.filter_index == 2

				elseif (poi_type == "stations" and sectorID ~= trainingID) then
					-- "Sol II System, Sector M-4 (Serco Guarded)|Planet: Vichama|Station: Vichama Orbital (Serco Mining)|"
					desc = "Station: "..FactionName[poi_data.faction].." "..poi_data.class --      sector_data.desc:match("Station:.*%(([%w ]+)%)")
					insert_poi = self.filter_index == 1 or self.filter_index == 3

				elseif (self.other_list[systemID][sectorID]) then
					desc = self.other_list[systemID][sectorID]
					insert_poi = self.filter_index == 1 or self.filter_index == 4
				end
				if (insert_poi) then
					local sector_loc = ShortLocationStr(sectorID):match("%w%-%d+$")
					local highlight = NavRoute.GetFinalDestination() == sectorID
					table.insert(self.poi_list, {sectorID, desc, sector_loc, highlight = highlight})
				end
			end
		end

		table.sort(self.poi_list, function(a, b)
			local A, B = a[2]:match("^Wormhole"), b[2]:match("^Wormhole")
			if (A and B) then
				return a[2] < b[2]
			elseif (A or B) then
				return A and true or B and false
			end
			A, B = a[2]:match("^Station"), b[2]:match("^Station")
			if (A and B) then
				return a[2] < b[2]
			elseif (A or B) then
				return A and true or B and false
			end

			return a[2] < b[2]
		end)

		targetlist.API.UpdateDisplay(self, self.poi_list)
	end
end

function T.Tab:PLUGINS_LOADED()
	if (isdeclared("targetlist") and targetlist.API) then
		if (targetlist.API.Version >= 2) then
			targetlist.API.RegisterTab(T.Tab)
		else
			print("\127FFFFFFThe MDI-TargetList API version is different than expected.  Points-of-Interest Tab was not loaded.\127o")
		end
	end
end

function T.Tab:PLAYER_ENTERED_GAME()
	if (not (isdeclared("targetlist") and targetlist.API)) then
		print("\127FFFFFFThe Points-of-Interest Tab requires MDI-TargetList v4.1+.\127o")
	end

	if (self.is_active) then
		self:SECTOR_CHANGED(nil, GetCurrentSectorid(), true)
	end
end


RegisterEvent(T.Tab, "TARGETLIST_TAB_CHANGED")
RegisterEvent(T.Tab, "SECTOR_CHANGED")
RegisterEvent(T.Tab, "PLUGINS_LOADED")
RegisterEvent(T.Tab, "PLAYER_ENTERED_GAME")
RegisterEvent(T.Tab, "NAVROUTE_ADD")
RegisterEvent(T.Tab, "NAVROUTE_CHANGED")
RegisterEvent(T.Tab, "NAVROUTE_UNDOLAST")
